@@ -64,9 +64,6 @@ static const char *_version = "0.1.10";
 static PyObject *rrdtool_OperationalError;
 static PyObject *rrdtool_ProgrammingError;
 
-static char **rrdtool_argv = NULL;
-static int    rrdtool_argc = 0;
-
 /**
  * PyRRD_DateTime_FromTS: convert UNIX timestamp (time_t)
  * to Python datetime object.
@@ -126,7 +123,7 @@ PyRRD_String_FromCF(enum cf_en cf)
  * @return Zero if the function succeeds, otherwise -1
  */
 static int
-convert_args(char *command, PyObject *args)
+convert_args(char *command, PyObject *args, char ***rrdtool_argv, int *rrdtool_argc)
 {
     PyObject *o, *lo;
     int i, j, args_count, argv_count, element_count;
@@ -148,39 +145,39 @@ convert_args(char *command, PyObject *args)
         }
     }
 
-    rrdtool_argv = PyMem_New(char *, element_count + 1);
+    *rrdtool_argv = PyMem_New(char *, element_count + 1);
 
-    if (rrdtool_argv == NULL)
+    if (*rrdtool_argv == NULL)
         return -1;
 
     for (i = 0; i < args_count; i++) {
         o = PyTuple_GET_ITEM(args, i);
 
         if (PyRRD_String_Check(o))
-            rrdtool_argv[++argv_count] = PyRRD_String_AS_STRING(o);
+            (*rrdtool_argv)[++argv_count] = PyRRD_String_AS_STRING(o);
         else if (PyList_CheckExact(o)) {
             for (j = 0; j < PyList_Size(o); j++) {
                 lo = PyList_GetItem(o, j);
 
                 if (PyRRD_String_Check(lo))
-                    rrdtool_argv[++argv_count] = PyRRD_String_AS_STRING(lo);
+                    (*rrdtool_argv)[++argv_count] = PyRRD_String_AS_STRING(lo);
                 else {
-                    PyMem_Del(rrdtool_argv);
+                    PyMem_Del(*rrdtool_argv);
                     PyErr_Format(PyExc_TypeError,
                       "Element %d in argument %d must be str", j, i);
                     return -1;
                 }
             }
         } else {
-            PyMem_Del(rrdtool_argv);
+            PyMem_Del(*rrdtool_argv);
             PyErr_Format(rrdtool_ProgrammingError,
               "Argument %d must be str or list of str", i);
             return -1;
         }
     }
 
-    rrdtool_argv[0] = command;
-    rrdtool_argc = element_count + 1;
+    (*rrdtool_argv)[0] = command;
+    *rrdtool_argc = element_count + 1;
 
     return 0;
 }
@@ -189,10 +186,10 @@ convert_args(char *command, PyObject *args)
  * Destroy argument vector.
  */
 static void
-destroy_args(void)
+destroy_args(char ***rrdtool_argv)
 {
-    PyMem_Del(rrdtool_argv);
-    rrdtool_argv = NULL;
+    PyMem_Del(*rrdtool_argv);
+    *rrdtool_argv = NULL;
 }
 
 /**
@@ -270,10 +267,12 @@ static char _rrdtool_create__doc__[] = "Create a new Round Robin Database.\n\n\
 static PyObject *
 _rrdtool_create(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int status;
 
-    if (convert_args("create", args) == -1)
+    if (convert_args("create", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -289,7 +288,7 @@ _rrdtool_create(PyObject *Py_UNUSED(self), PyObject *args)
         ret = Py_None;
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -307,10 +306,12 @@ static char _rrdtool_dump__doc__[] = "Dump an RRD to XML.\n\n\
 static PyObject *
 _rrdtool_dump(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int status;
 
-    if (convert_args("dump", args) == -1)
+    if (convert_args("dump", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -326,7 +327,7 @@ _rrdtool_dump(PyObject *Py_UNUSED(self), PyObject *args)
         ret = Py_None;
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -344,10 +345,12 @@ static char _rrdtool_update__doc__[] = "Store a new set of values into\
 static PyObject *
 _rrdtool_update(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int status;
 
-    if (convert_args("update", args) == -1)
+    if (convert_args("update", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -363,7 +366,7 @@ _rrdtool_update(PyObject *Py_UNUSED(self), PyObject *args)
         ret = Py_None;
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -375,10 +378,12 @@ static char _rrdtool_updatev__doc__[] = "Store a new set of values into "\
 static PyObject *
 _rrdtool_updatev(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     rrd_info_t *data;
 
-    if (convert_args("updatev", args) == -1)
+    if (convert_args("updatev", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -394,7 +399,7 @@ _rrdtool_updatev(PyObject *Py_UNUSED(self), PyObject *args)
         rrd_info_free(data);
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -414,6 +419,8 @@ static char _rrdtool_fetch__doc__[] = "Fetch data from an RRD.\n\n\
 static PyObject *
 _rrdtool_fetch(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret, *range_tup, *dsnam_tup, *data_list, *t;
     rrd_value_t *data, *datai, dv;
     unsigned long step, ds_cnt, i, j, row;
@@ -421,7 +428,7 @@ _rrdtool_fetch(PyObject *Py_UNUSED(self), PyObject *args)
     char **ds_namv;
     int status;
 
-    if (convert_args("fetch", args) == -1)
+    if (convert_args("fetch", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -473,7 +480,7 @@ _rrdtool_fetch(PyObject *Py_UNUSED(self), PyObject *args)
 
     rrd_freemem(ds_namv);
     rrd_freemem(data);
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -489,10 +496,12 @@ static char _rrdtool_flushcached__doc__[] = "Flush RRD files from memory.\n\n\
 static PyObject *
 _rrdtool_flushcached(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int status;
 
-    if (convert_args("flushcached", args) == -1)
+    if (convert_args("flushcached", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -508,7 +517,7 @@ _rrdtool_flushcached(PyObject *Py_UNUSED(self), PyObject *args)
         ret = Py_None;
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -581,12 +590,14 @@ static char _rrdtool_graph__doc__[] = "Create a graph based on one or more " \
 static PyObject *
 _rrdtool_graph(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int xsize, ysize, i, status;
     double ymin, ymax;
-    char **calcpr;
+    char **calcpr = NULL;
 
-    if (convert_args("graph", args) == -1)
+    if (convert_args("graph", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -623,7 +634,7 @@ _rrdtool_graph(PyObject *Py_UNUSED(self), PyObject *args)
     }
 
     rrd_freemem(calcpr);
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -637,10 +648,12 @@ static char _rrdtool_graphv__doc__[] = "Create a graph based on one or more " \
 static PyObject *
 _rrdtool_graphv(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     rrd_info_t *data;
 
-    if (convert_args("graphv", args) == -1)
+    if (convert_args("graphv", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -656,7 +669,7 @@ _rrdtool_graphv(PyObject *Py_UNUSED(self), PyObject *args)
         rrd_info_free(data);
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -680,6 +693,8 @@ static char _rrdtool_xport__doc__[] = "Dictionary representation of data " \
 static PyObject *
 _rrdtool_xport(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int xsize, status;
     char **legend_v;
@@ -687,7 +702,7 @@ _rrdtool_xport(PyObject *Py_UNUSED(self), PyObject *args)
     unsigned long step, col_cnt;
     rrd_value_t *data, *datai;
 
-    if (convert_args("xport", args) == -1)
+    if (convert_args("xport", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -759,7 +774,7 @@ _rrdtool_xport(PyObject *Py_UNUSED(self), PyObject *args)
         rrd_freemem(data);
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
 
     return ret;
 }
@@ -776,10 +791,12 @@ static char _rrdtool_list__doc__[] = "List RRDs in storage.\n\n" \
 static PyObject *
 _rrdtool_list(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret, *str;
     char *data, *ptr, *end;
 
-    if (convert_args("list", args) == -1)
+    if (convert_args("list", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -816,7 +833,7 @@ _rrdtool_list(PyObject *Py_UNUSED(self), PyObject *args)
         rrd_freemem(data);
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -836,10 +853,12 @@ static char _rrdtool_tune__doc__[] = "Modify some basic properties of a " \
 static PyObject *
 _rrdtool_tune(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int status;
 
-    if (convert_args("tune", args) == -1)
+    if (convert_args("tune", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -855,7 +874,7 @@ _rrdtool_tune(PyObject *Py_UNUSED(self), PyObject *args)
         ret = Py_None;
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -872,10 +891,12 @@ static char _rrdtool_first__doc__[] = "Get the first UNIX timestamp of the "\
 static PyObject *
 _rrdtool_first(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int ts;
 
-    if (convert_args("first", args) == -1)
+    if (convert_args("first", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -889,7 +910,7 @@ _rrdtool_first(PyObject *Py_UNUSED(self), PyObject *args)
     } else
         ret = PyRRD_Int_FromLong((long) ts);
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -905,10 +926,12 @@ static char _rrdtool_last__doc__[] = "Get the UNIX timestamp of the most "\
 static PyObject *
 _rrdtool_last(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int ts;
 
-    if (convert_args("last", args) == -1)
+    if (convert_args("last", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -922,7 +945,7 @@ _rrdtool_last(PyObject *Py_UNUSED(self), PyObject *args)
     } else
         ret = PyRRD_Int_FromLong((long) ts);
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -940,10 +963,12 @@ static char _rrdtool_resize__doc__[] = "Modify the number of rows in a "\
 static PyObject *
 _rrdtool_resize(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     int status;
 
-    if (convert_args("resize", args) == -1)
+    if (convert_args("resize", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -959,7 +984,7 @@ _rrdtool_resize(PyObject *Py_UNUSED(self), PyObject *args)
         ret = Py_None;
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -976,10 +1001,12 @@ static char _rrdtool_info__doc__[] = "Extract header information from an "\
 static PyObject *
 _rrdtool_info(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret;
     rrd_info_t *data;
 
-    if (convert_args("info", args) == -1)
+    if (convert_args("info", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
 
     Py_BEGIN_ALLOW_THREADS
@@ -995,7 +1022,7 @@ _rrdtool_info(PyObject *Py_UNUSED(self), PyObject *args)
         rrd_info_free(data);
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
     return ret;
 }
 
@@ -1011,13 +1038,15 @@ static char _rrdtool_lastupdate__doc__[] = "Returns datetime and value stored "\
 static PyObject *
 _rrdtool_lastupdate(PyObject *Py_UNUSED(self), PyObject *args)
 {
+    char **rrdtool_argv = NULL;
+    int    rrdtool_argc = 0;
     PyObject *ret, *ds_dict, *lastupd;
     int status;
     time_t last_update;
     char **ds_names, **last_ds;
     unsigned long ds_cnt, i;
 
-    if (convert_args("lastupdate", args) == -1)
+    if (convert_args("lastupdate", args, &rrdtool_argv, &rrdtool_argc) == -1)
         return NULL;
     else if (rrdtool_argc < 2) {
         PyErr_SetString(rrdtool_ProgrammingError, "Missing filename argument");
@@ -1064,7 +1093,9 @@ _rrdtool_lastupdate(PyObject *Py_UNUSED(self), PyObject *args)
             }
 
             PyDict_SetItemString(ds_dict, ds_names[i], val);
-            Py_DECREF(val);
+            
+            if (val != Py_None)
+                Py_DECREF(val);
 
             free(last_ds[i]);
             free(ds_names[i]);
@@ -1075,7 +1106,7 @@ _rrdtool_lastupdate(PyObject *Py_UNUSED(self), PyObject *args)
 
     }
 
-    destroy_args();
+    destroy_args(&rrdtool_argv);
 
     return ret;
 }
@@ -1200,7 +1231,7 @@ _rrdtool_fetch_cb_wrapper(
         *ds_namv = (char **)calloc(*ds_cnt, sizeof(char *));
 
         if (*ds_namv == NULL) {
-            rrd_set_error("an error occured while allocating memory for "
+            rrd_set_error("an error occurred while allocating memory for "
                 "ds_namv when allocating memory for python callback");
             goto gil_release_err;
         }
