@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.7.0, Copyright 1997-2017 by Tobi Oetiker
+ * RRDtool 1.7.1, Copyright 1997-2019 by Tobi Oetiker
  *****************************************************************************
  * rrd_first Return
  *****************************************************************************
@@ -91,7 +91,7 @@ time_t rrd_first_r(
     rrd_file_t *rrd_file;
 
     rrd_init(&rrd);
-    rrd_file = rrd_open(filename, &rrd, RRD_READONLY);
+    rrd_file = rrd_open(filename, &rrd, RRD_READONLY | RRD_LOCK);
     if (rrd_file == NULL) {
         goto err_free;
     }
