@@ -1,10 +1,10 @@
 /*****************************************************************************
- * RRDtool 1.6.0, Copyright 1997-2016 by Tobi Oetiker
+ * RRDtool 1.7.1, Copyright 1997-2019 by Tobi Oetiker
  *****************************************************************************
  * rrd_format.c  RRD Database Format helper functions
  *****************************************************************************/
 #include "rrd_tool.h"
-#ifdef WIN32
+#ifdef _WIN32
 #include "stdlib.h"
 #endif
 
@@ -27,7 +27,7 @@ enum dst_en dst_conv(
 }
 
 
-enum cf_en cf_conv(
+enum cf_en rrd_cf_conv(
     const char *string)
 {
 
