@@ -1,10 +1,14 @@
 /****************************************************************************
- * RRDtool 1.7.0, Copyright 1997-2017 by Tobi Oetiker
+ * RRDtool 1.7.1, Copyright 1997-2019 by Tobi Oetiker
  ****************************************************************************
  * rrd_graph_helper.c  commandline parser functions
  *                     this code initially written by Alex van den Bogaerdt
  ****************************************************************************/
 
+#ifdef __MINGW64__
+#define __USE_MINGW_ANSI_STDIO 1    /* for %lli */
+#endif
+
 #include <locale.h>
 #include "rrd_config.h"
 #ifdef HAVE_STDINT_H
@@ -541,7 +545,7 @@ static graph_desc_t* newGraphDescription(image_desc_t *const im,enum gf_en gf,pa
     double linewidth = 1;
     char *t,*x;
     if ((t=getKeyValueArgument("linewidth",1,pa))&&(*t!=0)) {
-      if ((getDouble(t,&linewidth,&x))||(linewidth<=0)) {
+      if ((getDouble(t,&linewidth,&x))||(linewidth<0)) {
 	rrd_set_error("Bad line width: %s",t); return NULL;
       }
     }
@@ -862,7 +866,7 @@ static int parse_textalign(enum gf_en,parsedargs_t*,image_desc_t *const);
 static int parse_shift(enum gf_en,parsedargs_t*,image_desc_t *const);
 static int parse_xport(enum gf_en,parsedargs_t*,image_desc_t *const);
 
-/* this is needed for LINE,AREA,STACk so that the labels get done correctly... */
+/* this is needed for LINE,AREA,STACK so that the labels get done correctly... */
 static void legend_shift(char *legend)
 {
   if (!legend || !legend[0]) { return; }
@@ -1455,7 +1459,11 @@ static int parse_shift(enum gf_en gf,parsedargs_t* pa,image_desc_t *const im) {
   if (gdp->shidx>=0) {
     dprintf("SHIFTBY : %s (%i)\n",im->gdes[gdp->shidx].vname,gdp->shidx);
   } else {
+#if defined _WIN32 && SIZEOF_TIME_T == 8    /* in case of __MINGW64__, _WIN64 and _MSC_VER >= 1400 (ifndef _USE_32BIT_TIME_T) */
+    dprintf("SHIFTBY : %lli\n",gdp->shval); /* argument 3 has type 'time_t {aka long long int}' */
+#else
     dprintf("SHIFTBY : %li\n",gdp->shval);
+#endif
   }
   dprintf("=================================\n");
   /* and return */
