@@ -1,5 +1,5 @@
 /*****************************************************************************
- * RRDtool 1.7.0, Copyright 1997-2017 by Tobi Oetiker
+ * RRDtool 1.7.1, Copyright 1997-2019 by Tobi Oetiker
  *****************************************************************************
  * rrd_info  Get Information about the configuration of an RRD
  *****************************************************************************/
@@ -164,7 +164,7 @@ rrd_info_t *rrd_info_r(
     enum dst_en current_ds;
 
     rrd_init(&rrd);
-    rrd_file = rrd_open(filename, &rrd, RRD_READONLY);
+    rrd_file = rrd_open(filename, &rrd, RRD_READONLY | RRD_LOCK);
     if (rrd_file == NULL)
         goto err_free;
 
