@@ -1,5 +1,5 @@
 /****************************************************************************
- * RRDtool 1.6.0, Copyright 1997-2016 by Tobi Oetiker
+ * RRDtool 1.7.1, Copyright 1997-2019 by Tobi Oetiker
  ****************************************************************************
  * rrd_create.h
  ****************************************************************************/
@@ -70,7 +70,7 @@ typedef int candidate_selectfunc_t(const rra_def_t *tofill, const rra_def_t *may
 
 candidate_t *find_candidate_rras(const rrd_t *rrd, const rra_def_t *rra, int *cnt,
 				 candidate_extra_t extra,
-                                 candidate_selectfunc_t);
+                                 candidate_selectfunc_t *select_func);
 
 #endif
 
