@@ -1,20 +1,27 @@
 /*****************************************************************************
- * RRDtool 1.7.0, Copyright 1997-2017 by Tobi Oetiker
+ * RRDtool 1.7.1, Copyright 1997-2019 by Tobi Oetiker
  *****************************************************************************
  * rrd_open.c  Open an RRD File
  *****************************************************************************
  * $Id$
  *****************************************************************************/
 
-#include "rrd_tool.h"
-#include "unused.h"
-
-#ifdef WIN32
-#include <stdlib.h>
-#include <fcntl.h>
-#include <sys/stat.h>
+#ifdef _WIN32
+#include <windows.h>
+#include <winsdkver.h>  /* Defines _WIN32_MAXVER */
+#if _WIN32_MAXVER >= 0x0602 /* _WIN32_WINNT_WIN8 */
+#include <synchapi.h>
 #endif
 
+#include <stdlib.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <sys/stat.h>
+#include <limits.h>
+#endif                          /* WIN32 */
+
+#include "rrd_tool.h"
+#include "unused.h"
 
 #ifdef HAVE_BROKEN_MS_ASYNC
 #include <sys/types.h>
@@ -27,12 +34,12 @@
 
 #define MEMBLK 8192
 
-#ifdef WIN32
-#define	_LK_UNLCK	0	/* Unlock */
-#define	_LK_LOCK	1	/* Lock */
-#define	_LK_NBLCK	2	/* Non-blocking lock */
-#define	_LK_RLCK	3	/* Lock for read only */
-#define	_LK_NBRLCK	4	/* Non-blocking lock for read only */
+#ifdef _WIN32
+#define	_LK_UNLCK	0   /* Unlock */
+#define	_LK_LOCK	1   /* Lock */
+#define	_LK_NBLCK	2   /* Non-blocking lock */
+#define	_LK_RLCK	3   /* "Same as _LK_NBLCK" */
+#define	_LK_NBRLCK	4   /* "Same as _LK_LOCK" */
 
 
 #define	LK_UNLCK	_LK_UNLCK
@@ -57,7 +64,7 @@
 	size_t wanted = sizeof(dst_t)*(cnt); \
 	if (offset + wanted > rrd_file->file_len) { \
 		rrd_set_error("reached EOF while loading header " #dst); \
-		goto out_nullify_head; \
+		goto out_close; \
 	} \
 	(dst) = (dst_t*)(void*) (data + offset); \
 	offset += wanted; \
@@ -68,12 +75,12 @@
         size_t got; \
 	if ((dst = (dst_t*)malloc(wanted)) == NULL) { \
 		rrd_set_error(#dst " malloc"); \
-		goto out_nullify_head; \
+		goto out_close; \
 	} \
         got = read (rrd_simple_file->fd, dst, wanted); \
 	if (got != wanted) { \
 		rrd_set_error("short read while reading header " #dst); \
-                goto out_nullify_head; \
+                goto out_close; \
 	} \
 	offset += got; \
     }
@@ -85,12 +92,12 @@
         size_t got; \
 	if ((dst = (dst_t*)malloc(wanted)) == NULL) { \
 		rrd_set_error(#dst " malloc"); \
-		goto out_nullify_head; \
+		goto out_close; \
 	} \
         got = rrd_rados_read(rrd_file->rados, dst, wanted, offset); \
 	if (got != wanted) { \
 		rrd_set_error("short read while reading header " #dst); \
-                goto out_nullify_head; \
+                goto out_close; \
 	} \
 	offset += got; \
     }
@@ -104,10 +111,10 @@
       __rrd_read_mmap(dst, dst_t, cnt) \
     }
 #elif defined(HAVE_LIBRADOS) && !defined(HAVE_MMAP)
-    if (rrd_file->rados) \
-      __rrd_read_rados(dst, dst_t, cnt) \
-    else \
-      __rrd_read_seq(dst, dst_t, cnt) \
+if (rrd_file->rados)
+    __rrd_read_rados(dst, dst_t, cnt)
+        else
+    __rrd_read_seq(dst, dst_t, cnt)
     }
 #elif defined(HAVE_MMAP)
 #define __rrd_read(dst, dst_t, cnt) \
@@ -124,6 +131,12 @@
 #endif
 #endif
 
+static int rrd_rwlock(
+    rrd_file_t *rrd_file,
+    int writelock);
+static int close_and_unlock(
+    int fd);
+
 /* Open a database file, return its header and an open filehandle,
  * positioned to the first cdp in the first rra.
  * In the error path of rrd_open, only rrd_free(&rrd) has to be called
@@ -150,12 +163,12 @@ rrd_file_t *rrd_open(
     struct stat statb;
     rrd_file_t *rrd_file = NULL;
     rrd_simple_file_t *rrd_simple_file = NULL;
-    size_t     newfile_size = 0;
-    size_t header_len, value_cnt, data_len;
+    size_t    newfile_size = 0;
 
     /* Are we creating a new file? */
-    if((rdwr & RRD_CREAT) && (rrd->stat_head != NULL))
-    {
+    if (rdwr & RRD_CREAT) {
+        size_t    header_len, value_cnt, data_len;
+
         header_len = rrd_get_header_size(rrd);
 
         value_cnt = 0;
@@ -166,23 +179,24 @@ rrd_file_t *rrd_open(
 
         newfile_size = header_len + data_len;
     }
-    
-    rrd_file = (rrd_file_t*)malloc(sizeof(rrd_file_t));
+
+    rrd_file = (rrd_file_t *) malloc(sizeof(rrd_file_t));
     if (rrd_file == NULL) {
         rrd_set_error("allocating rrd_file descriptor for '%s'", file_name);
         return NULL;
     }
     memset(rrd_file, 0, sizeof(rrd_file_t));
     rrd_file->rrd = rrd;
-    
+
     rrd_file->pvt = malloc(sizeof(rrd_simple_file_t));
-    if(rrd_file->pvt == NULL) {
+    if (rrd_file->pvt == NULL) {
         rrd_set_error("allocating rrd_simple_file for '%s'", file_name);
         free(rrd_file);
         return NULL;
     }
     memset(rrd_file->pvt, 0, sizeof(rrd_simple_file_t));
-    rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
+    rrd_simple_file = (rrd_simple_file_t *) rrd_file->pvt;
+    rrd_simple_file->fd = -1;
 
 #ifdef DEBUG
     if ((rdwr & (RRD_READONLY | RRD_READWRITE)) ==
@@ -196,14 +210,22 @@ rrd_file_t *rrd_open(
 
 #ifdef HAVE_LIBRADOS
     if (strncmp("ceph//", file_name, 6) == 0) {
-      rrd_file->rados = rrd_rados_open(file_name + 6);
-      if (rrd_file->rados == NULL)
-          goto out_free;
+        rrd_file->rados = rrd_rados_open(file_name + 6);
+        if (rrd_file->rados == NULL)
+            goto out_free;
 
-      if (rdwr & RRD_CREAT)
-          goto out_done;
+        if (rdwr & RRD_LOCK) {
+            /* Note: rados read lock is not implemented.  See rrd_lock(). */
+            if (rrd_rwlock(rrd_file, rdwr & RRD_READWRITE) != 0) {
+                rrd_set_error("could not lock RRD");
+                goto out_close;
+            }
+        }
 
-      goto read_check;
+        if (rdwr & RRD_CREAT)
+            goto out_done;
+
+        goto read_check;
     }
 #endif
 
@@ -219,16 +241,16 @@ rrd_file_t *rrd_open(
         rrd_simple_file->mm_flags = MAP_PRIVATE;
 # endif
 # ifdef MAP_NORESERVE
-        rrd_simple_file->mm_flags |= MAP_NORESERVE;  /* readonly, so no swap backing needed */
+        rrd_simple_file->mm_flags |= MAP_NORESERVE; /* readonly, so no swap backing needed */
 # endif
 #endif
     } else {
         if (rdwr & RRD_READWRITE) {
             flags |= O_RDWR;
-#ifdef HAVE_MMAP 
-            rrd_simple_file->mm_flags = MAP_SHARED; 
-            rrd_simple_file->mm_prot |= PROT_WRITE; 
-#endif 
+#ifdef HAVE_MMAP
+            rrd_simple_file->mm_flags = MAP_SHARED;
+            rrd_simple_file->mm_prot |= PROT_WRITE;
+#endif
         }
         if (rdwr & RRD_CREAT) {
             flags |= (O_CREAT | O_TRUNC);
@@ -239,35 +261,58 @@ rrd_file_t *rrd_open(
     }
     if (rdwr & RRD_READAHEAD) {
 #ifdef MAP_POPULATE
-        rrd_simple_file->mm_flags |= MAP_POPULATE;   /* populate ptes and data */
+        rrd_simple_file->mm_flags |= MAP_POPULATE;  /* populate ptes and data */
 #endif
 #if defined MAP_NONBLOCK
-        rrd_simple_file->mm_flags |= MAP_NONBLOCK;   /* just populate ptes */
+        rrd_simple_file->mm_flags |= MAP_NONBLOCK;  /* just populate ptes */
 #endif
     }
 #if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)
     flags |= O_BINARY;
 #endif
 
+#if defined(_WIN32)
+    /* In Windows we need FILE_SHARE_DELETE, so that the file can be
+     * renamed/replaced later on in rrd_create.c
+     * This is only possible using CreateFileA() first and not using open() alone */
+    HANDLE    handle;
+
+    handle =
+        CreateFileA(file_name, GENERIC_READ | GENERIC_WRITE,
+                    FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+                    NULL, OPEN_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+    if ((rrd_simple_file->fd = _open_osfhandle((intptr_t) handle, flags)) < 0) {
+        rrd_set_error("opening '%s': %s", file_name, rrd_strerror(errno));
+        goto out_free;
+    }
+#else
     if ((rrd_simple_file->fd = open(file_name, flags, 0666)) < 0) {
         rrd_set_error("opening '%s': %s", file_name, rrd_strerror(errno));
         goto out_free;
     }
+#endif
 
 #ifdef HAVE_MMAP
 #ifdef HAVE_BROKEN_MS_ASYNC
-    if (rdwr & RRD_READWRITE) {    
+    if (rdwr & RRD_READWRITE) {
         /* some unices, the files mtime does not get updated
            on memory mapped files, in order to help them,     
-           we update the the timestamp at this point.      
+           we update the timestamp at this point.      
            The thing happens pretty 'close' to the open    
            call so the chances of a race should be minimal.    
-                
-           Maybe ask your vendor to fix your OS ... */    
-           utime(file_name,NULL);  
+
+           Maybe ask your vendor to fix your OS ... */
+        utime(file_name, NULL);
     }
-#endif    
 #endif
+#endif
+
+    if (rdwr & RRD_LOCK) {
+        if (rrd_rwlock(rrd_file, rdwr & RRD_READWRITE) != 0) {
+            rrd_set_error("could not lock RRD");
+            goto out_close;
+        }
+    }
 
     /* Better try to avoid seeks as much as possible. stat may be heavy but
      * many concurrent seeks are even worse.  */
@@ -280,88 +325,70 @@ rrd_file_t *rrd_open(
     } else {
         rrd_file->file_len = newfile_size;
 #ifdef HAVE_POSIX_FALLOCATE
-	/* man: posix_fallocate() returns zero on success,
-	 * or an error number on failure.  Note that errno is not set.
-	 */
-	int fret = posix_fallocate(rrd_simple_file->fd, 0, newfile_size);
+        /* man: posix_fallocate() returns zero on success,
+         * or an error number on failure.  Note that errno is not set.
+         */
+        int       fret =
+            posix_fallocate(rrd_simple_file->fd, 0, newfile_size);
         if (fret) {
             rrd_set_error("posix_fallocate '%s': %s", file_name,
                           rrd_strerror(fret));
             goto out_close;
         } else {
-            goto no_lseek_necessary;        
+            goto no_lseek_necessary;
         }
 #endif
         if (lseek(rrd_simple_file->fd, newfile_size - 1, SEEK_SET) == -1) {
             rrd_set_error("lseek '%s': %s", file_name, rrd_strerror(errno));
             goto out_close;
         }
-        if ( write(rrd_simple_file->fd, "\0", 1) == -1){    /* poke */
+        if (write(rrd_simple_file->fd, "\0", 1) == -1) {    /* poke */
             rrd_set_error("write '%s': %s", file_name, rrd_strerror(errno));
             goto out_close;
         }
-        if (lseek(rrd_simple_file->fd, 0, SEEK_SET) == -1){
+        if (lseek(rrd_simple_file->fd, 0, SEEK_SET) == -1) {
             rrd_set_error("lseek '%s': %s", file_name, rrd_strerror(errno));
             goto out_close;
         }
     }
-    no_lseek_necessary:
-#if !defined(HAVE_MMAP) && defined(HAVE_POSIX_FADVISE)
-    /* In general we need no read-ahead when dealing with rrd_files.
-       When we stop reading, it is highly unlikely that we start up again.
-       In this manner we actually save time and disk access (and buffer cache).
-       Thanks to Dave Plonka for the Idea of using POSIX_FADV_RANDOM here. */
-    posix_fadvise(rrd_simple_file->fd, 0, 0, POSIX_FADV_RANDOM);
-#endif
-
-/*
-        if (rdwr & RRD_READWRITE)
-        {
-           if (setvbuf((rrd_simple_file->fd),NULL,_IONBF,2)) {
-                  rrd_set_error("failed to disable the stream buffer\n");
-                  return (-1);
-           }
-        }
-*/
+  no_lseek_necessary:
 
 #ifdef HAVE_MMAP
 #ifndef HAVE_POSIX_FALLOCATE
-	/* force allocating the file on the underlying filesystem to prevent any
-	 * future bus error when the filesystem is full and attempting to write
-	 * trough the file mapping. Filling the file using memset on the file
-	 * mapping can also lead some bus error, so we use the old fashioned
-	 * write().
-	 */
+    /* force allocating the file on the underlying filesystem to prevent any
+     * future bus error when the filesystem is full and attempting to write
+     * trough the file mapping. Filling the file using memset on the file
+     * mapping can also lead some bus error, so we use the old fashioned
+     * write().
+     */
     if (rdwr & RRD_CREAT) {
-		char     buf[4096];
-		unsigned i;
+        char      buf[4096];
+        unsigned  i;
 
-		memset(buf, DNAN, sizeof buf);
-		lseek(rrd_simple_file->fd, offset, SEEK_SET);
-        
-		for (i = 0; i < (newfile_size - 1) / sizeof buf; ++i)
-		{
-			if (write(rrd_simple_file->fd, buf, sizeof buf) == -1)
-			{
-				rrd_set_error("write '%s': %s", file_name, rrd_strerror(errno));
-				goto out_close;
-			}
-		}
-		
-		if (write(rrd_simple_file->fd, buf,
-					(newfile_size - 1) % sizeof buf) == -1)
-		{
-			rrd_set_error("write '%s': %s", file_name, rrd_strerror(errno));
-			goto out_close;
-		}
+        memset(buf, DNAN, sizeof buf);
+        lseek(rrd_simple_file->fd, offset, SEEK_SET);
 
-		lseek(rrd_simple_file->fd, 0, SEEK_SET);
+        for (i = 0; i < (newfile_size - 1) / sizeof buf; ++i) {
+            if (write(rrd_simple_file->fd, buf, sizeof buf) == -1) {
+                rrd_set_error("write '%s': %s", file_name,
+                              rrd_strerror(errno));
+                goto out_close;
+            }
+        }
+
+        if (write(rrd_simple_file->fd, buf,
+                  (newfile_size - 1) % sizeof buf) == -1) {
+            rrd_set_error("write '%s': %s", file_name, rrd_strerror(errno));
+            goto out_close;
+        }
+
+        lseek(rrd_simple_file->fd, 0, SEEK_SET);
     }
 #endif
 
-    data = mmap(0, rrd_file->file_len, 
-        rrd_simple_file->mm_prot, rrd_simple_file->mm_flags,
-        rrd_simple_file->fd, offset);
+    data = mmap(0, rrd_file->file_len,
+                rrd_simple_file->mm_prot, rrd_simple_file->mm_flags,
+                rrd_simple_file->fd, offset);
 
     /* lets see if the first read worked */
     if (data == MAP_FAILED) {
@@ -370,24 +397,41 @@ rrd_file_t *rrd_open(
         goto out_close;
     }
     rrd->__mmap_start = data;
-    rrd->__mmap_size  = rrd_file->file_len;
-	    
+    rrd->__mmap_size = rrd_file->file_len;
+
     rrd_simple_file->file_start = data;
 #endif
     if (rdwr & RRD_CREAT)
         goto out_done;
+
+    if (rdwr & RRD_READAHEAD) {
+        /* If perfect READAHEAD is not achieved for whatever reason, caller
+           will not thank us for advising the kernel of RANDOM access below. */
+        rdwr |= RRD_COPY;
+    }
+    /* In general we need no read-ahead when dealing with rrd_files.
+       When we stop reading, it is highly unlikely that we start up again.
+       In this manner we actually save time and disk access (and buffer cache).
+       Thanks to Dave Plonka for the Idea of using POSIX_FADV_RANDOM here. */
 #ifdef USE_MADVISE
     if (rdwr & RRD_COPY) {
         /* We will read everything in a moment (copying) */
-        madvise(data, rrd_file->file_len, MADV_SEQUENTIAL );
+        madvise(data, rrd_file->file_len, MADV_SEQUENTIAL);
     } else {
         /* We do not need to read anything in for the moment */
         madvise(data, rrd_file->file_len, MADV_RANDOM);
     }
 #endif
+#if !defined(HAVE_MMAP) && defined(HAVE_POSIX_FADVISE)
+    if (rdwr & RRD_COPY) {
+        posix_fadvise(rrd_simple_file->fd, 0, 0, POSIX_FADV_SEQUENTIAL);
+    } else {
+        posix_fadvise(rrd_simple_file->fd, 0, 0, POSIX_FADV_RANDOM);
+    }
+#endif
 
 #ifdef HAVE_LIBRADOS
-read_check:
+  read_check:
 #endif
 
     __rrd_read(rrd->stat_head, stat_head_t,
@@ -396,12 +440,12 @@ read_check:
     /* lets do some test if we are on track ... */
     if (memcmp(rrd->stat_head->cookie, RRD_COOKIE, sizeof(RRD_COOKIE)) != 0) {
         rrd_set_error("'%s' is not an RRD file", file_name);
-        goto out_nullify_head;
+        goto out_close;
     }
 
     if (rrd->stat_head->float_cookie != FLOAT_COOKIE) {
         rrd_set_error("This RRD was created on another architecture");
-        goto out_nullify_head;
+        goto out_close;
     }
 
     version = atoi(rrd->stat_head->version);
@@ -409,7 +453,7 @@ read_check:
     if (version > atoi(RRD_VERSION5)) {
         rrd_set_error("can't handle RRD file version %s",
                       rrd->stat_head->version);
-        goto out_nullify_head;
+        goto out_close;
     }
     __rrd_read(rrd->ds_def, ds_def_t,
                rrd->stat_head->ds_cnt);
@@ -443,53 +487,75 @@ read_check:
     rrd_file->header_len = offset;
     rrd_file->pos = offset;
 
-    {
-      unsigned long row_cnt = 0;
-
-      for (ui=0; ui<rrd->stat_head->rra_cnt; ui++)
-        row_cnt += rrd->rra_def[ui].row_cnt;
-
-      size_t  correct_len = rrd_file->header_len +
-        sizeof(rrd_value_t) * row_cnt * rrd->stat_head->ds_cnt;
-
-#ifdef HAVE_LIBRADOS
-      /* skip length checking for rados file */
-      if (rrd_file->rados) {
-        rrd_file->file_len = correct_len;
-      }
+#if defined(HAVE_MMAP) && defined(USE_MADVISE)
+    if (data != MAP_FAILED) {
+        /* MADV_SEQUENTIAL mentions drop-behind.  Override it for the header
+         * now we've read it, in case anyone implemented drop-behind.
+         *
+         * Do *not* fall back to fadvise() for !HAVE_MMAP.  In that case,
+         * we've copied the header and will not read it again.  Doing e.g.
+         * FADV_NORMAL on Linux (4.12) on *any* region would negate the
+         * effect of previous FADV_SEQUENTIAL.
+         */
+        madvise(data, sysconf(_SC_PAGESIZE), MADV_NORMAL);
+        madvise(data, sysconf(_SC_PAGESIZE), MADV_WILLNEED);
+    }
 #endif
 
-      if (correct_len > rrd_file->file_len)
-      {
-        rrd_set_error("'%s' is too small (should be %ld bytes)",
-                      file_name, (long long) correct_len);
-        goto out_nullify_head;
-      }
-      if (rdwr & RRD_READVALUES) {
-	  long d_offset = offset;
-	  
-	  __rrd_read(rrd->rrd_value, rrd_value_t,
-		     row_cnt * rrd->stat_head->ds_cnt);
+    {
+        unsigned long row_cnt = 0;
 
-	  rrd_file->header_len = d_offset;
-	  rrd_file->pos = d_offset;
-      }
-      
+        for (ui = 0; ui < rrd->stat_head->rra_cnt; ui++)
+            row_cnt += rrd->rra_def[ui].row_cnt;
+
+        size_t    correct_len = rrd_file->header_len +
+            sizeof(rrd_value_t) * row_cnt * rrd->stat_head->ds_cnt;
+
+#ifdef HAVE_LIBRADOS
+        /* skip length checking for rados file */
+        if (rrd_file->rados) {
+            rrd_file->file_len = correct_len;
+        }
+#endif
+
+        if (correct_len > rrd_file->file_len) {
+            rrd_set_error("'%s' is too small (should be %ld bytes)",
+                          file_name, (long long) correct_len);
+            goto out_close;
+        }
+        if (rdwr & RRD_READVALUES) {
+            __rrd_read(rrd->rrd_value, rrd_value_t,
+                       row_cnt * rrd->stat_head->ds_cnt);
+
+            if (rrd_seek(rrd_file, rrd_file->header_len, SEEK_SET) != 0)
+                goto out_close;
+        }
     }
 
-    
-    
   out_done:
     return (rrd_file);
-  out_nullify_head:
-    rrd->stat_head = NULL;
+
   out_close:
 #ifdef HAVE_MMAP
     if (data != MAP_FAILED)
-      munmap(data, rrd_file->file_len);
+        munmap(data, rrd_file->file_len);
 #endif
+#ifdef HAVE_LIBRADOS
+    if (rrd_file->rados)
+        rrd_rados_close(rrd_file->rados);
+#endif
+    if (rrd_simple_file->fd >= 0) {
+        /* keep the original error */
+        char     *e = strdup(rrd_get_error());
 
-    close(rrd_simple_file->fd);
+        close_and_unlock(rrd_simple_file->fd);
+
+        if (e) {
+            rrd_set_error(e);
+            free(e);
+        } else
+            rrd_set_error("error message was lost (out of memory)");
+    }
   out_free:
     free(rrd_file->pvt);
     free(rrd_file);
@@ -505,10 +571,11 @@ void mincore_print(
     char *mark)
 {
     rrd_simple_file_t *rrd_simple_file;
-    rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
+
+    rrd_simple_file = (rrd_simple_file_t *) rrd_file->pvt;
 #ifdef HAVE_MMAP
     /* pretty print blocks in core */
-    size_t     off;
+    size_t    off;
     unsigned char *vec;
     ssize_t   _page_size = sysconf(_SC_PAGESIZE);
 
@@ -517,7 +584,8 @@ void mincore_print(
     vec = malloc(off);
     if (vec != NULL) {
         memset(vec, 0, off);
-        if (mincore(rrd_simple_file->file_start, rrd_file->file_len, vec) == 0) {
+        if (mincore(rrd_simple_file->file_start, rrd_file->file_len, vec) ==
+            0) {
             int       prev;
             unsigned  is_in = 0, was_in = 0;
 
@@ -552,38 +620,147 @@ void mincore_print(
  */
 int rrd_lock(
     rrd_file_t *rrd_file)
+{
+    return rrd_rwlock(rrd_file, 1);
+}
+
+#if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)
+#define USE_WINDOWS_LOCK 1
+#endif
+
+#ifdef USE_WINDOWS_LOCK
+static
+int rrd_windows_lock(
+    int fd)
+{
+    int       ret;
+    long      pos;
+
+    /*
+     * _locking() is relative to fd position.
+     * We need to consistently lock bytes starting from 0,
+     * so we can successfully unlock on close.
+     *
+     * Note rrd_lock() API doesn't set a specific error message.
+     * Knowing that rrd_lock() (or even rrd_open()) failed should
+     * be specific enough, if someone manages to invoke rrdtool
+     * on something silly like a named pipe or COM1.
+     */
+    pos = tell(fd);
+    if (pos < 0)
+        return -1;
+
+    if (lseek(fd, 0, SEEK_SET) < 0)
+        return -1;
+
+    while (1) {
+        ret = _locking(fd, _LK_NBLCK, LONG_MAX);
+        if (ret == 0)
+            break;      /* success */
+        if (errno != EACCES)
+            break;      /* failure */
+        /* EACCES: someone else has the lock. */
+
+        /*
+         * Wait 0.01 seconds before trying again.  _locking()
+         * with _LK_LOCK would work similarly but waits 1 second
+         * between tries, which seems less desirable.
+         */
+        Sleep(10);
+    }
+
+    /* restore saved fd position */
+    if (lseek(fd, pos, SEEK_SET) < 0)
+        return -1;
+
+    return ret;
+}
+#endif
+
+static
+int close_and_unlock(
+    int fd)
+{
+    int       ret = 0;
+
+#ifdef USE_WINDOWS_LOCK
+    /*
+     * "If a process closes a file that has outstanding locks, the locks are
+     *  unlocked by the operating system. However, the time it takes for the
+     *  operating system to unlock these locks depends upon available system
+     *  resources. Therefore, it is recommended that your process explicitly
+     *  unlock all files it has locked when it terminates."  (?!)
+     */
+
+    if (lseek(fd, 0, SEEK_SET) < 0) {
+        rrd_set_error("lseek: %s", rrd_strerror(errno));
+        ret = -1;
+        goto out_close;
+    }
+
+    ret = _locking(fd, LK_UNLCK, LONG_MAX);
+    if (ret != 0 && errno == EACCES)
+        /* fd was not locked - this is entirely possible, ignore the error */
+        ret = 0;
+
+    if (ret != 0)
+        rrd_set_error("unlock file: %s", rrd_strerror(errno));
+  out_close:
+#endif
+
+    if (close(fd) != 0) {
+        ret = -1;
+        rrd_set_error("closing file: %s", rrd_strerror(errno));
+    }
+
+    return ret;
+}
+
+static
+int rrd_rwlock(
+    rrd_file_t *rrd_file,
+    int writelock)
 {
 #ifdef DISABLE_FLOCK
-    (void)rrd_file;
+    (void) rrd_file;
     return 0;
 #else
 #ifdef HAVE_LIBRADOS
-    if (rrd_file->rados)
-      return rrd_rados_lock(rrd_file->rados);
+    if (rrd_file->rados) {
+        /*
+         * No read lock on rados.  It would be complicated by the
+         * use of a short finite lock duration in rrd_rados_lock().
+         * Also rados does not provide blocking locks.
+         *
+         * Rados users may use snapshots if they need to
+         * e.g. obtain a consistent backup.
+         */
+        if (writelock)
+            return rrd_rados_lock(rrd_file->rados);
+        else
+            return 0;
+    }
 #endif
     int       rcstat;
     rrd_simple_file_t *rrd_simple_file;
-    rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
-    {
-#if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)
-        struct _stat st;
 
-        if (_fstat(rrd_simple_file->fd, &st) == 0) {
-            rcstat = _locking(rrd_simple_file->fd, _LK_NBLCK, st.st_size);
-        } else {
-            rcstat = -1;
-        }
+    rrd_simple_file = (rrd_simple_file_t *) rrd_file->pvt;
+#ifdef USE_WINDOWS_LOCK
+    /* _locking() does not support read locks; we always take a write lock */
+    rcstat = rrd_windows_lock(rrd_simple_file->fd);
 #else
+    {
         struct flock lock;
 
-        lock.l_type = F_WRLCK;  /* exclusive write lock */
+        lock.l_type = writelock ? F_WRLCK : /* exclusive write lock or */
+            F_RDLCK;    /* shared read lock */
         lock.l_len = 0; /* whole file */
         lock.l_start = 0;   /* start of file */
         lock.l_whence = SEEK_SET;   /* end of file */
 
         rcstat = fcntl(rrd_simple_file->fd, F_SETLK, &lock);
-#endif
     }
+#endif
 
     return (rcstat);
 #endif
@@ -596,20 +773,21 @@ void rrd_dontneed(
     rrd_t *rrd)
 {
     rrd_simple_file_t *rrd_simple_file;
+
 #if defined USE_MADVISE || defined HAVE_POSIX_FADVISE
-    size_t dontneed_start;
-    size_t rra_start;
-    size_t active_block;
-    size_t i;
+    size_t    dontneed_start;
+    size_t    rra_start;
+    size_t    active_block;
+    size_t    i;
     ssize_t   _page_size = sysconf(_SC_PAGESIZE);
 
     if (rrd_file == NULL) {
 #if defined DEBUG && DEBUG
-	    fprintf (stderr, "rrd_dontneed: Argument 'rrd_file' is NULL.\n");
+        fprintf(stderr, "rrd_dontneed: Argument 'rrd_file' is NULL.\n");
 #endif
-	    return;
+        return;
     }
-    rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
+    rrd_simple_file = (rrd_simple_file_t *) rrd_file->pvt;
 
 #if defined DEBUG && DEBUG > 1
     mincore_print(rrd_file, "before");
@@ -624,7 +802,7 @@ void rrd_dontneed(
                        + rrd->rra_ptr[i].cur_row
                        * rrd->stat_head->ds_cnt * sizeof(rrd_value_t));
         if (active_block > dontneed_start) {
-#ifdef USE_MADVISE 
+#ifdef USE_MADVISE
             madvise(rrd_simple_file->file_start + dontneed_start,
                     active_block - dontneed_start - 1, MADV_DONTNEED);
 #else
@@ -651,13 +829,13 @@ void rrd_dontneed(
 
     if (dontneed_start < rrd_file->file_len) {
 #ifdef USE_MADVISE
-	    madvise(rrd_simple_file->file_start + dontneed_start,
-		    rrd_file->file_len - dontneed_start, MADV_DONTNEED);
+        madvise(rrd_simple_file->file_start + dontneed_start,
+                rrd_file->file_len - dontneed_start, MADV_DONTNEED);
 #else
 #ifdef HAVE_POSIX_FADVISE
-	    posix_fadvise(rrd_simple_file->fd, dontneed_start,
-			  rrd_file->file_len - dontneed_start,
-			  POSIX_FADV_DONTNEED);
+        posix_fadvise(rrd_simple_file->fd, dontneed_start,
+                      rrd_file->file_len - dontneed_start,
+                      POSIX_FADV_DONTNEED);
 #endif
 #endif
     }
@@ -676,26 +854,30 @@ int rrd_close(
     rrd_file_t *rrd_file)
 {
     rrd_simple_file_t *rrd_simple_file;
-    rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
-    int       ret;
 
-#ifdef HAVE_MMAP
-    ret = munmap(rrd_simple_file->file_start, rrd_file->file_len);
-    if (ret != 0)
-        rrd_set_error("munmap rrd_file: %s", rrd_strerror(errno));
-#endif
+    rrd_simple_file = (rrd_simple_file_t *) rrd_file->pvt;
+    int       ret = 0;
+
 #ifdef HAVE_LIBRADOS
-    if (rrd_file->rados)
-        ret = rrd_rados_close(rrd_file->rados);
-    else
+    if (rrd_file->rados) {
+        if (rrd_rados_close(rrd_file->rados) != 0)
+            ret = -1;
+    }
 #endif
-    ret = close(rrd_simple_file->fd);
-
-    if (ret != 0)
-        rrd_set_error("closing file: %s", rrd_strerror(errno));
+#ifdef HAVE_MMAP
+    if (rrd_simple_file->file_start != NULL) {
+        if (munmap(rrd_simple_file->file_start, rrd_file->file_len) != 0) {
+            ret = -1;
+            rrd_set_error("munmap rrd_file: %s", rrd_strerror(errno));
+        }
+    }
+#endif
+    if (rrd_simple_file->fd >= 0) {
+        if (close_and_unlock(rrd_simple_file->fd) != 0)
+            ret = -1;
+    }
     free(rrd_file->pvt);
     free(rrd_file);
-    rrd_file = NULL;
     return ret;
 }
 
@@ -708,19 +890,21 @@ off_t rrd_seek(
     int whence)
 {
 #ifdef HAVE_LIBRADOS
-  /* no seek for rados */
-  if (rrd_file->rados) {
-    rrd_file->pos = off;
-    return 0;
-  }
+    /* no seek for rados */
+    if (rrd_file->rados) {
+        rrd_file->pos = off;
+        return 0;
+    }
 #endif
 
     off_t     ret = 0;
+
 #ifndef HAVE_MMAP
     rrd_simple_file_t *rrd_simple_file;
-    rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
+
+    rrd_simple_file = (rrd_simple_file_t *) rrd_file->pvt;
 #endif
- 
+
 #ifdef HAVE_MMAP
     if (whence == SEEK_SET)
         rrd_file->pos = off;
@@ -758,13 +942,15 @@ ssize_t rrd_read(
 {
 #ifdef HAVE_LIBRADOS
     if (rrd_file->rados) {
-        ssize_t ret = rrd_rados_read(rrd_file->rados, buf, count, rrd_file->pos);
+        ssize_t   ret =
+            rrd_rados_read(rrd_file->rados, buf, count, rrd_file->pos);
         if (ret > 0)
             rrd_file->pos += ret;
         return ret;
     }
 #endif
-    rrd_simple_file_t *rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
+    rrd_simple_file_t *rrd_simple_file = (rrd_simple_file_t *) rrd_file->pvt;
+
 #ifdef HAVE_MMAP
     size_t    _cnt = count;
     ssize_t   _surplus;
@@ -805,23 +991,27 @@ ssize_t rrd_write(
 {
 #ifdef HAVE_LIBRADOS
     if (rrd_file->rados) {
-      size_t ret = rrd_rados_write(rrd_file->rados, buf, count, rrd_file->pos);
-      if (ret > 0)
-        rrd_file->pos += count;
-      return ret;
+        size_t    ret =
+            rrd_rados_write(rrd_file->rados, buf, count, rrd_file->pos);
+        if (ret > 0)
+            rrd_file->pos += count;
+        return ret;
     }
 #endif
-    rrd_simple_file_t *rrd_simple_file = (rrd_simple_file_t *)rrd_file->pvt;
+    rrd_simple_file_t *rrd_simple_file = (rrd_simple_file_t *) rrd_file->pvt;
+
 #ifdef HAVE_MMAP
-    size_t old_size = rrd_file->file_len;
+    size_t    old_size = rrd_file->file_len;
+
     if (count == 0)
         return 0;
     if (buf == NULL)
         return -1;      /* EINVAL */
-    
-    if((rrd_file->pos + count) > old_size)
-    {
-        rrd_set_error("attempting to write beyond end of file (%ld + %ld > %ld)",rrd_file->pos, count, old_size);
+
+    if ((rrd_file->pos + count) > old_size) {
+        rrd_set_error
+            ("attempting to write beyond end of file (%ld + %ld > %ld)",
+             rrd_file->pos, count, old_size);
         return -1;
     }
     /* can't use memcpy since the areas overlap when tuning */
@@ -865,30 +1055,35 @@ void rrd_init(
 
 
 /* free RRD data, act correctly, regardless of mmap'ped or malloc'd memory. */
-static void free_rrd_ptr_if_not_mmapped(void *m, const rrd_t *rrd)
+static void free_rrd_ptr_if_not_mmapped(
+    void *m,
+    const rrd_t *rrd)
 {
-    if (m == NULL) return;
-    
+    if (m == NULL)
+        return;
+
     if (rrd == NULL || rrd->__mmap_start == NULL) {
-	free(m);
-	return;
+        free(m);
+        return;
     }
-    
+
     /* is this ALWAYS correct on all supported platforms ??? */
-    long ofs = (char*)m - (char*)rrd->__mmap_start;
+    long      ofs = (char *) m - (char *) rrd->__mmap_start;
+
     if (ofs < rrd->__mmap_size) {
-	// DO NOT FREE, this memory is mmapped!!
-	return;
+        // DO NOT FREE, this memory is mmapped!!
+        return;
     }
-    
+
     free(m);
 }
 
 void rrd_free(
     rrd_t *rrd)
 {
-    if (rrd == NULL) return;
-    
+    if (rrd == NULL)
+        return;
+
     free_rrd_ptr_if_not_mmapped(rrd->live_head, rrd);
     rrd->live_head = NULL;
     free_rrd_ptr_if_not_mmapped(rrd->stat_head, rrd);
@@ -939,8 +1134,7 @@ void rrd_notify_row(
 unsigned long rrd_select_initial_row(
     rrd_file_t UNUSED(*rrd_file),
     int UNUSED(rra_idx),
-    rra_def_t *rra
-    )
+    rra_def_t *rra)
 {
     return rrd_random() % rra->row_cnt;
 }
