@@ -2,7 +2,7 @@
 #define RRD_TOOL_H_3853987DDF7E4709A5B5849E5A6204F4
 
 /*****************************************************************************
- * RRDtool 1.7.0, Copyright 1997-2017 by Tobi Oetiker
+ * RRDtool 1.7.1, Copyright 1997-2019 by Tobi Oetiker
  *****************************************************************************
  * rrd_tool.h   Common Header File
  *****************************************************************************/
@@ -14,7 +14,7 @@ extern    "C" {
 
 #include "rrd.h"
 
-#if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__)
+#if defined(_WIN32) && !defined(__CYGWIN__) && !defined(__CYGWIN32__) && !defined(__MINGW32__)
 
 /* Win32 only includes */
 
@@ -133,6 +133,7 @@ typedef int (*rrd_fetch_cb_t)(
 #define RRD_COPY        (1<<4)
 #define RRD_EXCL        (1<<5)
 #define RRD_READVALUES  (1<<6)
+#define RRD_LOCK        (1<<7)
 
     enum cf_en rrd_cf_conv(
     const char *string);
