@@ -27,7 +27,7 @@
  *
  * Revision 1.3  2001/03/07 21:21:54  oetiker
  * complete rewrite of rrdgraph documentation. This also includes info
- * on upcomming/planned changes to the rrdgraph interface and functionality
+ * on upcoming/planned changes to the rrdgraph interface and functionality
  * -- Alex van den Bogaerdt <alex@slot.hollandcasino.nl>
  *
  * Revision 1.2  2001/03/04 13:01:55  oetiker
@@ -116,6 +116,7 @@ int rrd_tune(
     struct optparse options;
     int opt;
 
+    rrd_thread_init();
     /* Fix CWE-457 */
     memset(&rrd, 0, sizeof(rrd_t));
 
@@ -173,7 +174,8 @@ int rrd_tune(
     }
 
     rrd_init(&rrd);
-    rrd_file = rrd_open(in_filename, &rrd, RRD_READWRITE | RRD_READAHEAD | RRD_READVALUES);
+    rrd_file = rrd_open(in_filename, &rrd, RRD_READWRITE | RRD_LOCK |
+                                           RRD_READAHEAD | RRD_READVALUES);
     if (rrd_file == NULL) {
 	goto done;
     }
@@ -405,7 +407,7 @@ int rrd_tune(
             }
     }
 
-    optind = handle_modify(&rrd, in_filename, options.argc, options.argv, options.optind + 1, opt_newstep);
+    options.optind = handle_modify(&rrd, in_filename, options.argc, options.argv, options.optind + 1, opt_newstep);
     rc = 0;
 done:
     if (in_filename && rrdc_is_any_connected()) {
@@ -415,11 +417,11 @@ done:
 	rrdc_forget(in_filename);
 	rrd_clear_error();
         
-        if (e && *e) {
+        if (e) {
             rrd_set_error(e);
-        }
-        if (e) free(e);
-        
+            free(e);
+        } else
+            rrd_set_error("error message was lost (out of memory)");
     }
     if (rrd_file) {
 	rrd_close(rrd_file);
