@@ -76,7 +76,8 @@ int rrd_xport(
         {0}
     };
 
-    rrd_graph_init(&im);
+    rrd_thread_init();
+    rrd_graph_init(&im, IMAGE_INIT_NO_CAIRO);
 
     rrd_parsetime("end-24h", &start_tv);
     rrd_parsetime("now", &end_tv);
@@ -587,7 +588,7 @@ static int rrd_xport_format_xmljson(int flags,stringbuffer_t *buffer,image_desc_
 
   /* define the time format */
   char* timefmt=NULL;
-  /* unfortunatley we have to do it this way,
+  /* unfortunately we have to do it this way,
      as when no --x-graph argument is given,
      then the xlab_user is not in a clean state (e.g. zero-filled) */
   if (im->xlab_user.minsec!=-1.0) { timefmt=im->xlab_user.stst; }
@@ -629,9 +630,9 @@ static int rrd_xport_format_xmljson(int flags,stringbuffer_t *buffer,image_desc_
     }
   } else {
     if (json) {
-      snprintf(buf,sizeof(buf),"    \"%s\": %lld,\n",META_START_TAG,(long long int)start);
+      snprintf(buf,sizeof(buf),"    \"%s\": %lld,\n",META_START_TAG,(long long int)start+step);
     } else {
-      snprintf(buf,sizeof(buf),"    <%s>%lld</%s>\n",META_START_TAG,(long long int)start,META_START_TAG);
+      snprintf(buf,sizeof(buf),"    <%s>%lld</%s>\n",META_START_TAG,(long long int)start+step,META_START_TAG);
     }
   }
   addToBuffer(buffer,buf,0);
@@ -732,7 +733,7 @@ static int rrd_xport_format_xmljson(int flags,stringbuffer_t *buffer,image_desc_
   }
   addToBuffer(buffer,buf,0);
   /* iterate over data */
-  for (time_t ti = start; ti < end; ti += step) {
+  for (time_t ti = start+step; ti <= end; ti += step) {
     if (timefmt) {
       struct tm loc;
       localtime_r(&ti,&loc);
@@ -791,7 +792,7 @@ static int rrd_xport_format_xmljson(int flags,stringbuffer_t *buffer,image_desc_
       ptr++;
     }
     if (json){
-      addToBuffer(buffer,(ti < end-(time_t)step ? " ],\n" : " ]\n"),0);
+      addToBuffer(buffer,(ti <= end-(time_t)step ? " ],\n" : " ]\n"),0);
     }
     else {
       snprintf(buf,sizeof(buf),"</%s>\n", DATA_ROW_TAG);
